#! /usr/bin/env ruby 
# output sequences specified by IDs in the fasta format from seqfile files.


seqfile = nil
id_file = nil
CMD = File::basename($0)

while true do
	k = 0
	case ARGV[k]
	when '-f', '-d'
		seqfile = ARGV[k + 1]
		ARGV.shift
		ARGV.shift
	when '-e'
		id_file = ARGV[k + 1]
		ARGV.shift
		ARGV.shift
	when '-h', '--help', /^-.*$/
		printf "#{CMD} [-f db_file] [-e entry_id_file] entry_id ...\n" 
		exit 0
	else
		break
	end
end

require("LIB/Utils/is_comment.rb")
require("LIB/Bio")
require("LIB/Bio/Sequence")

include Bio
#include Bio::Sequence


def non_del_pos(entries, *seqs)
	seqs = seqs.flatten
	seqs = seqs.map do |seq|
	  if seq =~ /^\/.*\/$/
		Regexp.new(seq[1...-1]) if seq =~ /^\/.*\/$/
	  else
		seq
	  end
	end
	entries.each_entry do |entry|
	  next if entry.entry_id.nil?	# for blank line
	  seqs.each do | e |
	    e_range = nil
	    seq = nil
	    definition = nil
	    if  e.is_a?(Regexp)
	      if entry.entry_id =~ e
		definition = entry.definition
		#printf "%s", entry.seq.to_fasta(definition, 60)
		seq = Bio::Sequence.auto(entry.seq)
	      end
	    else
	      e_id, e_range, = e.split(':')
	      if entry.entry_id == e_id
		definition = entry.definition
		if e_range.nil?
		  #printf "%s", entry.seq.to_fasta(definition, 60)
		  seq = Bio::Sequence.auto(entry.seq)
		else
		  if e_range =~ /complement/
		  	seq = Bio::Sequence::NA.new(entry.seq)
		  else
		  	seq = Bio::Sequence.auto(entry.seq)
		  end
		  seq = seq.splicing(e_range)

		  #if ! seq.nil?
		    definition = entry.definition + ' [range=' + e_range + ']'
		    #printf "%s", seq.to_fasta(definition, 60)
		  #end
	        end
	      end
	    end
	    if ! seq.nil?
		    pos_str = nil
		    pos_start = nil
		    pos_end = nil
		    (0...seq.length).each do |i|
			if seq[i] != '-' and seq[i] != '.'
				pos_start = i + 1 if pos_start.nil?
			   	pos_end = i + 1
			else
				if ! pos_start.nil?
					if pos_start >= pos_end
						pos = sprintf ",%d", pos_start
					else
						pos = sprintf ",%d..%d", pos_start, pos_end
					end
					if pos_str.nil?
						pos_str = pos[1..-1]
					else
						pos_str = pos_str + pos
					end
				end
		    		pos_start = nil
		    		pos_end = nil
			end
		    end
	 	if ! pos_start.nil? and pos_start > 1
					if pos_start >= pos_end
						pos = sprintf ",%d", pos_start
					else
						pos = sprintf ",%d..%d", pos_start, pos_end
					end
					if pos_str.nil?
						pos_str = pos[1..-1]
					else
						pos_str = pos_str + pos
					end
		end
	    	
		if (pos_str.nil? and ! pos_start.nil?) or ! pos_str.nil?
		  if e_range.nil?
		      printf "%s", entry.entry_id
		  else
		      printf "%s:%s", entry.entry_id,e_range
		  end
		  if ! pos_str.nil?
		    printf ":join(%s)\n", pos_str
		  else
		    printf "\n"
	    	  end
		end
	    end
	  end
	end
end

if seqfile.nil?
	seqfile = $stdin
end
entries = Bio::FlatFile.auto(seqfile)

cchar = "#"

if ! id_file.nil?
	id_file = File.open(id_file)
	seqs = []
        id_file.each_line { |l|
                l.chomp!
                if ! is_comment?(l, cchar) then
                        seqs.push(l)
                end
         } 
	id_file.close
	non_del_pos(entries, seqs)
elsif ARGV.size > 0 then
	non_del_pos(entries, ARGV)
else
	raise "DB file or entry file must be given as a option." if seqfile == $stdin
	#port = IO.popen("remove_comments","r")
	port = $stdin
	begin
		seqs = []
                port.each_line { |l|
                        l.chomp!
                        if ! is_comment?(l, cchar) then
                                seqs.push(l)
                        end
                }
		non_del_pos(entries, seqs)
	ensure
		port.close
	end
end


