#! /usr/local/bin/ruby

# pick a subtree from a large tree, which includes the specified sequences.

require 'set'

require "LIB/Phylogeny"
include Phylogeny

	cchar = '#'

	leave_or_rm = ARGV[1].to_s		# leave or remove
	if leave_or_rm != "leave" and  leave_or_rm != "rm"
		raise "The secand parameter must be either 'leave' or 'rm'."
	end

	newickfile = ARGV[0].to_s
	port_full = File.open(newickfile)

	if ARGV.size > 2 
		otufile = ARGV[2].to_s
		port_otu = File.open(otufile)
	else 
		port_otu = $stdin
	end

	otus = Set.new 
	port_otu.each_line do |l|
		l.chomp!
		if ! is_comment?(l, cchar) then
			otus.add(l)
		end
	end

	full_str = get_tree_str_from_file(port_full, "#")
	full_tree = make_tree_from_newick_str(full_str)
	printf $stderr, "#otu = %d\n", full_tree.all_leaf_nodes.size

	#otu = seed_tree.all_leaf_nodes

	nodes_removed = full_tree.rm_unnecessary_nodes
	if ! nodes_removed.nil?
		printf $stderr, "The %d unnecessary nodes were removed.\n", nodes_removed.size
	end

	leaves = full_tree.all_leaf_nodes
	if leave_or_rm == "rm" then
	  leaves.each do | l |
		otu_id = l.node_info[0]
		if otus.include?(otu_id) then
			full_tree.delete_linked_subtree(l)
		end
	  end
	elsif leave_or_rm == "leave"
	  leaves.each do | l |
		otu_id = l.node_info[0]
		if ! otus.include?(otu_id) then
			full_tree.delete_linked_subtree(l)
		end
	  end
	end

	nodes_removed = full_tree.rm_unnecessary_nodes
	if ! nodes_removed.nil?
		printf $stderr, "The %d unnecessary nodes were removed.\n", nodes_removed.size
	end

	print_tree_in_newick(full_tree, "",  " ")

	printf $stderr, "becomes #otu = %d\n", full_tree.all_leaf_nodes.size

